==> digikam-1.0.0.tar.bz2: digikam-1.0.0/utilities/advancedrename/common/highlighter.cpp <==
/* ============================================================
 *
 * This file is a part of digiKam project
 * http://www.digikam.org
 *
 * Date        : 2009-11-19
 * Description : syntax highlighter for AdvancedRename utility
 *
 * Copyright (C) 2009 by Andi Clemens <andi dot clemens at gmx dot net>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include "highlighter.h"

// Qt includes

#include <QFileInfo>
#include <QTextEdit>

// Local includes

#include "parser.h"

namespace Digikam
{

Highlighter::Highlighter(QTextEdit* parent, Parser* _parser)
           : QSyntaxHighlighter(parent), parser(_parser)

{
    setupHighlightingGrammar();
}

Highlighter::~Highlighter()
{
}

void Highlighter::highlightBlock(const QString& text)
{
    foreach (const HighlightingRule &rule, highlightingRules)
    {
        QRegExp expression(rule.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);

            switch (rule.type)
            {
                case OptionPattern:
                case ModifierPattern:
                {
                    // highlight parameters in options and modifiers
                    if (expression.numCaptures() > 0 && !expression.cap(1).isEmpty())
                    {
                        QString fullmatched  = expression.cap(0);
                        QString parameters   = expression.cap(1);
                        if (parameters.startsWith(':'))
                        {
                            parameters.remove(0, 1);
                            if (!parameters.isEmpty())
                            {
                                int pindex = fullmatched.indexOf(parameters);
                                while (pindex >= 0)
                                {
                                    int plength = parameters.length();
                                    setFormat(index + pindex, plength, parameterFormat);
                                    pindex = fullmatched.indexOf(parameters, pindex + plength);
                                }
                            }
                        }
                    }

                    if (expression.numCaptures() > 0)
                    {
                        // highlight quoted text in options and modifiers
                        QRegExp quotationExp = quotationRule.pattern;
                        QString fullmatched  = expression.cap(0);
                        int qindex           = quotationExp.indexIn(fullmatched);
                        while (qindex >= 0)
                        {
                            int qlength = quotationExp.matchedLength();
                            setFormat(index + qindex, qlength, quotationFormat);
                            qindex = quotationExp.indexIn(fullmatched, qindex + qlength);
                        }
                    }
                }
                default: break;
            }
            index = expression.indexIn(text, index + length);
        }
    }

    // mark invalid modifiers in the parse string
    ParseSettings settings;
    settings.parseString = text;
    ParseResults invalid = parser->invalidModifiers(settings);
    foreach (const ParseResults::ResultsKey& key, invalid.keys())
    {
        setFormat(key.first, key.second, errorFormat);
    }
}

void Highlighter::setupHighlightingGrammar()
{
    if (!parser)
    {
        return;
    }

    HighlightingRule rule;

    // --------------------------------------------------------

    optionFormat.setForeground(Qt::red);

    foreach (Option* option, parser->options())
    {
        QRegExp r    = option->regExp();
        rule.type    = OptionPattern;
        rule.pattern = r;
        rule.format  = optionFormat;
        highlightingRules.append(rule);
    }

    // --------------------------------------------------------

    modifierFormat.setForeground(Qt::darkGreen);

    foreach (Modifier* modifier, parser->modifiers())
    {
        QRegExp r    = modifier->regExp();
        rule.type    = ModifierPattern;
        rule.pattern = r;
        rule.format  = modifierFormat;
        highlightingRules.append(rule);
    }

    // --------------------------------------------------------

    quotationFormat.setForeground(Qt::blue);
    quotationFormat.setFontItalic(true);
    quotationRule.pattern = QRegExp("\".*\"");
    quotationRule.pattern.setMinimal(true);
    quotationRule.format = quotationFormat;
    quotationRule.type   = QuotedTextPattern;

    // --------------------------------------------------------

    parameterFormat.setForeground(Qt::darkYellow);
    parameterFormat.setFontItalic(true);

    // --------------------------------------------------------

    errorFormat.setForeground(Qt::white);
    errorFormat.setBackground(Qt::red);
}

} // namespace Digikam

==> digikam-1.1.0.tar.bz2: digikam-1.1.0/utilities/advancedrename/common/highlighter.cpp <==
/* ============================================================
 *
 * This file is a part of digiKam project
 * http://www.digikam.org
 *
 * Date        : 2009-11-19
 * Description : syntax highlighter for AdvancedRename utility
 *
 * Copyright (C) 2009 by Andi Clemens <andi dot clemens at gmx dot net>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include "highlighter.h"

// Qt includes

#include <QFileInfo>
#include <QTextEdit>

// Local includes

#include "parser.h"

namespace Digikam
{

Highlighter::Highlighter(QTextEdit* parent, Parser* _parser)
           : QSyntaxHighlighter(parent), parser(_parser)

{
    setupHighlightingGrammar();
}

Highlighter::~Highlighter()
{
}

void Highlighter::highlightBlock(const QString& text)
{
    foreach (const HighlightingRule &rule, highlightingRules)
    {
        QRegExp expression(rule.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);

            switch (rule.type)
            {
                case OptionPattern:
                case ModifierPattern:
                {
                    // highlight parameters in options and modifiers
                    if (expression.numCaptures() > 0 && !expression.cap(1).isEmpty())
                    {
                        QString fullmatched  = expression.cap(0);
                        QString parameters   = expression.cap(1);
                        if (parameters.startsWith(':'))
                        {
                            parameters.remove(0, 1);
                            if (!parameters.isEmpty())
                            {
                                int pindex = fullmatched.indexOf(parameters);
                                while (pindex >= 0)
                                {
                                    int plength = parameters.length();
                                    setFormat(index + pindex, plength, parameterFormat);
                                    pindex = fullmatched.indexOf(parameters, pindex + plength);
                                }
                            }
                        }
                    }
                }
                default: break;
            }
            index = expression.indexIn(text, index + length);
        }
    }

    // mark invalid modifiers in the parse string
    ParseSettings settings;
    settings.parseString = text;
    ParseResults invalid = parser->invalidModifiers(settings);
    foreach (const ParseResults::ResultsKey& key, invalid.keys())
    {
        setFormat(key.first, key.second, errorFormat);
    }

    // highlight quoted text in options and modifiers
    {
        QRegExp expression(quotationRule.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            QString fullmatched  = expression.cap(0);
            int qlength = expression.matchedLength();
            setFormat(index, qlength, quotationFormat);
            index = expression.indexIn(text, index + qlength);
        }
    }
}

void Highlighter::setupHighlightingGrammar()
{
    if (!parser)
    {
        return;
    }

    HighlightingRule rule;

    // --------------------------------------------------------

    optionFormat.setForeground(Qt::red);

    foreach (Option* option, parser->options())
    {
        QRegExp r    = option->regExp();
        rule.type    = OptionPattern;
        rule.pattern = r;
        rule.format  = optionFormat;
        highlightingRules.append(rule);
    }

    // --------------------------------------------------------

    modifierFormat.setForeground(Qt::darkGreen);

    foreach (Modifier* modifier, parser->modifiers())
    {
        QRegExp r    = modifier->regExp();
        rule.type    = ModifierPattern;
        rule.pattern = r;
        rule.format  = modifierFormat;
        highlightingRules.append(rule);
    }

    // --------------------------------------------------------

    quotationFormat.setForeground(Qt::blue);
    quotationFormat.setFontItalic(true);
    quotationRule.pattern = QRegExp("\".*\"");
    quotationRule.pattern.setMinimal(true);
    quotationRule.format = quotationFormat;
    quotationRule.type   = QuotedTextPattern;

    // --------------------------------------------------------

    parameterFormat.setForeground(Qt::darkYellow);
    parameterFormat.setFontItalic(true);

    // --------------------------------------------------------

    errorFormat.setForeground(Qt::white);
    errorFormat.setBackground(Qt::red);
}

} // namespace Digikam
